/*
 *	uht_prx example 1
 *	Author : mcidclan
 */

#include <pspdisplay.h>
#include <malloc.h>
#include <Uht.h>
#include <psppower.h>
#include <pspctrl.h>
#include "addm33.h"

PSP_MODULE_INFO("uhtp", 0x1000, 1, 1);
PSP_NO_CREATE_MAIN_THREAD();
PSP_HEAP_SIZE_KB(1024);

vec2i out;
SceUID thid = -1;
SceModule *mdURef;
Uht *myUht = NULL;

SceUID cs_mgr_thid = -1;
SceUID cs_mgr_semaId;
u8 cs_mgr_workNum = 0;

void *bwork;

int SMHandler(SceModule2 *CMStart);

void cs_mgr_workByNum(u8 num){

	cs_mgr_workNum = num;
	sceKernelSignalSema(cs_mgr_semaId, 1);
}

int cs_mgr_thread(SceSize ags, void *agp){
	
	sceKernelDelayThread(6000000);
	// Allocate 0x22240 bytes for the working buffer
	bwork = malloc(0x22240);

	while(1){

		sceKernelWaitSema(cs_mgr_semaId , 1, 0);

		switch(cs_mgr_workNum){

			case 1: case 5 :

				myUht = new Uht();
				// Direct Render = true,  // true to use FB_565
				myUht->initBuffer(bwork, true, true);
				
				myUht->initCam(0x40);
				
				if(cs_mgr_workNum == 5){

					sceKernelDeleteSema(cs_mgr_semaId);
					sceKernelExitDeleteThread(0);
				} break;

			case 2: 
				delete myUht;
				myUht = NULL;
				break;

			case 3: myUht->relativeClean = true;
			
			case 4: 
				delete myUht;
				free(bwork);

				if(thid>=0)
				sceKernelTerminateDeleteThread(thid);
				sceKernelExitDeleteThread(0);			
		}

		cs_mgr_workNum = 0;
		sceKernelDelayThread(1);
	}
}


bool checkVsh(){

	if(!sceKernelFindModuleByName("VshControl")){

		cs_mgr_workByNum(5);
		return false;
	}

	return true;
 }

int main_thread(SceSize ags, void *agp){

	sceKernelDelayThread(7000000);
	sctrlHENSetStartModuleHandler(SMHandler);

	bool isVsh = checkVsh();

	void *cframe = NULL;
	int bufW, pixF, j, i = 0;
	u32 p0, p1;

	while(1){

		if(myUht != NULL && mdURef == NULL){

			getAbsPosition(myUht->pOut, &out);

			sceKernelDcacheWritebackAll();
			sceDisplayWaitVblankStart();
			sceDisplayGetFrameBuf(&cframe, &bufW, &pixF, 0);

			if(cframe != NULL){

				p0 = p1 = i = 0;
				while(p1<SIZE_FBu8){//WW*HH

					if(!pixF) ((u16*)cframe)[p0+i] = myUht->FB_565[p1+i];
					else ((u32*)cframe)[p0+i] = myUht->FB_8888[p1+i];
					
					i++;
					if(i == WW) {
						i = 0;
						p0+=bufW;
						p1+=WW;
					}
				}

				i=myUht->pOut.x;
				while(i<myUht->pOut.y){
					if(!pixF) ((u16*)cframe)[i+out.y*bufW] = 0xFF;
					else ((u32*)cframe)[i+out.y*bufW] = 0xFF0000FF;
					i++;
				}

				j=myUht->pOut.z;
				while(j<myUht->pOut.w){
					p0 = j*bufW;
					if(!pixF) ((u16*)cframe)[p0+out.x] = 0xFF;
					else ((u32*)cframe)[p0+out.x] = 0xFF0000FF;
					j++;
				}
			}
		}

		if(isVsh){

			mdURef = sceKernelFindModuleByName("sceUSB_Stor_Driver");
			if(myUht == NULL && mdURef == NULL) cs_mgr_workByNum(1);
			else if(myUht != NULL && mdURef != NULL) cs_mgr_workByNum(2);

		}

		sceKernelDelayThread(1);
	}

	return 0;
}

int SMHandler(SceModule2 *CMStart){

	char *cmname = CMStart->modname;
	if (strcmp(cmname, "camera_plugin_module") == 0) cs_mgr_workByNum(3);
	if (strcmp(cmname, "skype_main_plugin") == 0) cs_mgr_workByNum(3);
	if (strcmp(cmname, "opening_plugin_module") == 0) cs_mgr_workByNum(4);

	return 0;
}

extern "C"{

	int module_start(SceSize ags, void *agp){

		cs_mgr_semaId = sceKernelCreateSema("cs_mgr_sema", 0, 2, 2, 0);

		cs_mgr_thid = sceKernelCreateThread("cs_mgr_thread", cs_mgr_thread, 0x01, 0x400*8, 0, NULL);
		if(cs_mgr_thid>=0) sceKernelStartThread(cs_mgr_thid, ags, agp);

		thid = sceKernelCreateThread("uhtp_thread", main_thread, 0x10, 0x400*32, 0, NULL);
		if(thid>=0) sceKernelStartThread(thid, ags, agp);
		return 0;
	}

	int module_stop(SceSize args, void *argp){
		return 0;
	}
	
	void* getModuleInfo(void){
		return (void *) &module_info;
	}
}

